==> webCDwriter-2.4.0.tar.bz2: webCDwriter-2.4.0/CDWserver/CDWserver.cpp <==
/*
 * @(#)CDWserver.cpp
 *
 * This file is part of webCDwriter - Network CD Writing.
 *
 * Copyright (C) 1999-2002 Jrg P. M. Haeger
 *
 * webCDwriter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * webCDwriter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with webCDwriter; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Jrg Haeger, 17.03.1999
 * eMail: Joerg.Haeger@Uni-Bielefeld.de
 * http://wwwhomes.uni-bielefeld.de/jhaeger/
 */

#include <sys/types.h>
#include <grp.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "BufferedReader.h"
#include "Config.h"
#include "HTTPServer.h"
#include "Log.h"
#include "Server.h"
#include "ServerSocket.h"
#include "Socket.h"
#include "Thread.h"

Config config;
Log log;

class CDWserver: public Thread {
	Socket *clientSocket;
public:
	CDWserver(Socket *clientSocket) {
		this->clientSocket = clientSocket;
	}

	~CDWserver() {
		delete clientSocket;
	}

	void run() {
		try {
			BufferedReader &reader = clientSocket->getInputStream();
			if (reader.ready(10000)) {
				HTTPServer server(*clientSocket);
				server.run();
			}
			else {
				Server server(clientSocket->getOutputStream());
				server.run();
			}
		} catch (Error *e) {
			log.put("Error: %s", e->toString());
			delete e;
		} catch (Exception *e) {
			log.put("Exception: %s", e->getMessage().getBytes());
			delete e;
		} catch (const char *str) {
			log.put("Catched: %s", str);
		}

		log.debug("threads", "terminated");
	}
};

void
sigAbrtHandler(int n)
{
	log.put("sigAbrtHandler: preparing for termination");
	_exit(0);
}

void
sigBusHandler(int n)
{
	log.put("sigBusHandler: preparing for termination");
	_exit(0);
}

void
sigPipeHandler(int n)
{
	log.put("sigPipeHandler: broken pipe");
}

void
sigSegvHandler(int n)
{
	log.put("sigSegvHandler: preparing for termination");
	_exit(0);
}

void
sigTermHandler(int n)
{
	log.put("sigTermHandler: preparing for termination");
	Server::terminate();
	exit(0);
}

int
main(int argc, char **argv)
{
	if (argc != 2 || strcmp(argv[1], "stdout") != 0)
		// go to the background
		if (fork() > 0)
			exit(0);

	// try to create a pid file
	const char *pidFileName = getenv("pidFile");
	if (pidFileName == NULL)
		pidFileName = "/var/run/CDWserver.pid";
	FILE *pidFile = fopen(pidFileName, "w");
	if (pidFile != NULL)
	{
		fprintf(pidFile, "%d\n", getpid());
		fclose(pidFile);
	}

	config.setPermissions();

	// try to change the group of this process
	initgroups(config.getUser(), config.getGid());
	setgid(config.getGid());

	// try to change the user of this process
	setuid(config.getUid());

	// the files created by CDWserver should be unreadable
	// to normal users
	umask(S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IWOTH | S_IXOTH);

	// fix problem with mkisofs
	unsetenv("HOME");

	// start logging
	log.start(argc == 2 && strcmp(argv[1], "stdout") == 0);

	Server::initStatic();
	signal(SIGABRT, sigAbrtHandler);
	signal(SIGBUS, sigBusHandler);
	signal(SIGPIPE, sigPipeHandler);
	signal(SIGSEGV, sigSegvHandler);
	signal(SIGTERM, sigTermHandler);

	ServerSocket serverSocket(config.getPortNo());

	for (int i = 1; ; i++) {
		log.putv("waiting for request...");
		Socket *clientSocket = serverSocket.accept();
		log.putv("Connection %d accepted", i);

		(new CDWserver(clientSocket))->start();
	}

	return 0;
}

==> webCDwriter-2.4.1.tar.bz2: webCDwriter-2.4.1/CDWserver/CDWserver.cpp <==
/*
 * @(#)CDWserver.cpp
 *
 * This file is part of webCDwriter - Network CD Writing.
 *
 * Copyright (C) 1999-2002 Jrg P. M. Haeger
 *
 * webCDwriter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * webCDwriter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with webCDwriter; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Jrg Haeger, 17.03.1999
 * eMail: Joerg.Haeger@Uni-Bielefeld.de
 * http://wwwhomes.uni-bielefeld.de/jhaeger/
 */

#include <sys/types.h>
#include <grp.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include "BufferedReader.h"
#include "Config.h"
#include "HTTPServer.h"
#include "Log.h"
#include "Server.h"
#include "ServerSocket.h"
#include "Socket.h"
#include "Thread.h"
#include "Version.h"

Config config;
Log log;

class CDWserver: public Thread {
	Socket *clientSocket;
public:
	CDWserver(Socket *clientSocket) {
		this->clientSocket = clientSocket;
	}

	~CDWserver() {
		delete clientSocket;
	}

	void run() {
		try {
			BufferedReader &reader = clientSocket->getInputStream();
			if (reader.ready(10000)) {
				HTTPServer server(*clientSocket);
				server.run();
			}
			else {
				Server server(clientSocket->getOutputStream());
				server.run();
			}
		} catch (Error *e) {
			log.put("Error: %s", e->toString());
			delete e;
		} catch (Exception *e) {
			log.put("Exception: %s", e->getMessage().getBytes());
			delete e;
		} catch (const char *str) {
			log.put("Catched: %s", str);
		}

		log.debug("threads", "terminated");
	}
};

void
sigAbrtHandler(int n) {
	log.put(1, "sigAbrtHandler: preparing for termination");
	_exit(0);
}

void
sigBusHandler(int n) {
	log.put(1, "sigBusHandler: preparing for termination");
	_exit(0);
}

void
sigPipeHandler(int n) {
	log.put(4, "sigPipeHandler: broken pipe");
}

void
sigSegvHandler(int n) {
	log.put(1, "sigSegvHandler: preparing for termination");
	_exit(0);
}

void
sigTermHandler(int n) {
	log.put(1, "sigTermHandler: preparing for termination");
	Server::terminate();
	exit(0);
}

int
main(int argc, char **args) {
	int logToStdOut = 0, foreground = 0;
	for (int i = 1; i < argc; i++)
		if (strcmp(args[i], "--fg") == 0)
			foreground = 1;
		else
		if (strcmp(args[i], "--stdout") == 0)
			logToStdOut = 1;
		else {
			printf("%s %s\n", Version.product, Version.version);
			printf("%s %s\n", Version.copyright, Version.author);
			printf("Usage: %s [options]\n", args[0]);
			printf("\t--fg\t\tdo not go to the background\n");
			printf("\t--stdout\twrite the log to stdout\n");
			printf("\n");
			return 0;
		}

	if (!foreground)
		// go to the background
		if (fork() > 0)
			exit(0);

	// try to create a pid file
	const char *pidFileName = getenv("pidFile");
	if (pidFileName == NULL)
		pidFileName = "/var/run/CDWserver.pid";
	FILE *pidFile = fopen(pidFileName, "w");
	if (pidFile != NULL) {
		fprintf(pidFile, "%d\n", getpid());
		fclose(pidFile);
	}

	config.setPermissions();
	config.prepareBinaries();

	// try to change the group of this process
	initgroups(config.getUser(), config.getGid());
	setgid(config.getGid());

	// try to change the user of this process
	setuid(config.getUid());

	// the files created by CDWserver should be unreadable
	// to normal users
	umask(S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IWOTH | S_IXOTH);

	// fix problem with mkisofs
	unsetenv("HOME");

	// start logging
	log.start(logToStdOut);
	time_t t;
	time(&t);
	log.put(1, "*** %s %s, %s",
		Version.product, Version.version, ctime(&t));
	log.put(1, "uid = %d, euid = %d, gid = %d, egid = %d",
		getuid(), geteuid(), getgid(), getegid());

	Server::initStatic();
	signal(SIGABRT, sigAbrtHandler);
	signal(SIGBUS, sigBusHandler);
	signal(SIGPIPE, sigPipeHandler);
	signal(SIGSEGV, sigSegvHandler);
	signal(SIGTERM, sigTermHandler);

	ServerSocket serverSocket(config.getPortNo());

	for (int i = 1; ; i++) {
		log.putv("waiting for request...");
		Socket *clientSocket = serverSocket.accept();
		log.putv("Connection %d accepted", i);

		(new CDWserver(clientSocket))->start();
	}

	return 0;
}
